<?php

namespace Laralum\Events\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;

class Event extends Model
{
    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'laralum_events';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'title', 'user_id', 'description',
        'start_date', 'start_time', 'end_date',
        'end_time', 'price', 'color',
        'place', 'public',
    ];

    /**
     * Return the event author.
     */
    public function user()
    {
        return $this->belongsTo('\Laralum\Users\Models\User');
    }

    /**
     * Return all the event users.
     */
    public function users()
    {
        return $this->belongsToMany('\Laralum\Users\Models\User', 'laralum_event_user');
    }

    /**
     * Returns true if the event have the specified user.
     *
     * @param mixed $user
     */
    public function hasUser($user)
    {
        return EventUser::where([
            'event_id' => $this->id,
            'user_id'  => $user->id,
        ])->first();
    }

    /**
     * Adds a user into the event.
     *
     * @param mixed $user
     */
    public function addUser($user)
    {
        if (!$this->hasUser($user)) {
            return EventUser::create([
                'event_id' => $this->id,
                'user_id'  => $user->id,
            ]);
        }

        return false;
    }

    /**
     * Adds users into the event.
     *
     * @param array $users
     */
    public function addUsers($users)
    {
        foreach ($users as $user) {
            $this->addUser($user);
        }

        return true;
    }

    /**
     * Deletes the specified event user.
     *
     * @param mixed $user
     */
    public function deleteUser($user)
    {
        if ($this->hasUser($user)) {
            return EventUser::where([
                'event_id' => $this->id,
                'user_id'  => $user->id,
            ])->first()->delete();
        }

        return false;
    }

    /**
     * Deletes the specified role users.
     *
     * @param array $users
     */
    public function deleteUsers($users)
    {
        foreach ($users as $user) {
            $this->deleteUser($user);
        }

        return true;
    }

    /**
     * Get Carbon start datetime.
     *
     * @return \Carbon\Carbon
     */
    public function startDatetime()
    {
        $date = explode('-', $this->start_date);
        $time = explode(':', $this->start_time);
        $start_datetime = Carbon::create($date[0], $date[1], $date[2], $time[0], $time[1]);

        return $start_datetime;
    }

    /**
     * Get Carbon end datetime.
     *
     * @return \Carbon\Carbon
     */
    public function endDatetime()
    {
        $date = explode('-', $this->end_date);
        $time = explode(':', $this->end_time);
        $end_datetime = Carbon::create($date[0], $date[1], $date[2], $time[0], $time[1]);

        return $end_datetime;
    }

    /**
     * Returns true if event has started or passed.
     *
     * @return bool
     */
    public function started()
    {
        return $this->startDatetime()->isPast();
    }

    /**
     * Returns true if event has finished.
     *
     * @return bool
     */
    public function finished()
    {
        return $this->endDatetime()->isPast();
    }
}
